@@ -3,11 +3,13 @@ Merge OpenType Layout tables (GDEF / GPOS / GSUB).
 """
 from __future__ import print_function, division, absolute_import
 from fontTools.misc.py23 import *
+from fontTools.misc.fixedTools import otRound
 from fontTools.misc import classifyTools
 from fontTools.ttLib.tables import otTables as ot
 from fontTools.ttLib.tables import otBase as otBase
 from fontTools.ttLib.tables.DefaultTable import DefaultTable
-from fontTools.varLib import builder
+from fontTools.varLib import builder, varStore
+from fontTools.varLib.varStore import VarStoreInstancer
 from functools import reduce
 
 
@@ -196,12 +198,13 @@ def _Lookup_PairPos_get_effective_value_pair(subtables, firstGlyph, secondGlyph)
 @AligningMerger.merger(ot.SinglePos)
 def merge(merger, self, lst):
 	self.ValueFormat = valueFormat = reduce(int.__or__, [l.ValueFormat for l in lst], 0)
-	assert valueFormat & ~0xF == 0, valueFormat
+	assert len(lst) == 1 or (valueFormat & ~0xF == 0), valueFormat
 
 	# If all have same coverage table and all are format 1,
 	if all(v.Format == 1 for v in lst) and all(self.Coverage.glyphs == v.Coverage.glyphs for v in lst):
 		self.Value = otBase.ValueRecord(valueFormat)
 		merger.mergeThings(self.Value, [v.Value for v in lst])
+		self.ValueFormat = self.Value.getFormat()
 		return
 
 	# Upgrade everything to Format=2
@@ -221,6 +224,9 @@ def merge(merger, self, lst):
 		for j,glyph in enumerate(glyphs):
 			if values[j] is not None: continue
 			# Fill in value from other subtables
+			# Note!!! This *might* result in behavior change if ValueFormat2-zeroedness
+			# is different between used subtable and current subtable!
+			# TODO(behdad) Check and warn if that happens?
 			v = _Lookup_SinglePos_get_effective_value(merger.lookup_subtables[i], glyph)
 			if v is None:
 				v = otBase.ValueRecord(valueFormat)
@@ -230,7 +236,8 @@ def merge(merger, self, lst):
 
 	# Merge everything else; though, there shouldn't be anything else. :)
 	merger.mergeObjects(self, lst,
-			    exclude=('Format', 'Coverage', 'ValueRecord', 'Value', 'ValueCount'))
+			    exclude=('Format', 'Coverage', 'Value', 'ValueCount'))
+	self.ValueFormat = reduce(int.__or__, [v.getFormat() for v in self.Value], 0)
 
 @AligningMerger.merger(ot.PairSet)
 def merge(merger, self, lst):
@@ -269,10 +276,13 @@ def merge(merger, self, lst):
 	merger.mergeLists(self.PairValueRecord, padded)
 
 def _PairPosFormat1_merge(self, lst, merger):
+	assert _all_equal([l.ValueFormat2 == 0 for l in lst if l.PairSet]), "Report bug against fonttools."
+
 	# Merge everything else; makes sure Format is the same.
 	merger.mergeObjects(self, lst,
 			    exclude=('Coverage',
-				     'PairSet', 'PairSetCount'))
+				     'PairSet', 'PairSetCount',
+				     'ValueFormat1', 'ValueFormat2'))
 
 	empty = ot.PairSet()
 	empty.PairValueRecord = []
@@ -355,38 +365,13 @@ def _ClassDef_calculate_Format(self, font):
 			fmt = 1
 	self.Format = fmt
 
-def _PairPosFormat2_merge(self, lst, merger):
-
-	merger.mergeObjects(self, lst,
-			    exclude=('Coverage',
-				     'ClassDef1', 'Class1Count',
-				     'ClassDef2', 'Class2Count',
-				     'Class1Record'))
-
-	# Align coverages
-	glyphs, _ = _merge_GlyphOrders(merger.font,
-				       [v.Coverage.glyphs for v in lst])
-	self.Coverage.glyphs = glyphs
-	glyphSet = set(glyphs)
-
-	# Currently, if the coverage of PairPosFormat2 subtables are different,
-	# we do NOT bother walking down the subtable list when filling in new
-	# rows for alignment.  As such, this is only correct if current subtable
-	# is the last subtable in the lookup.  Ensure that.
-	#
-	# Note that our canonicalization process merges some PairPosFormat2's,
-	# so in reality this might not be common.
-	#
-	# TODO: Remove this requirement
-	for l,subtables in zip(lst,merger.lookup_subtables):
-		if l.Coverage.glyphs != glyphs:
-			assert l == subtables[-1]
+def _PairPosFormat2_align_matrices(self, lst, font, transparent=False):
 
 	matrices = [l.Class1Record for l in lst]
 
 	# Align first classes
-	self.ClassDef1, classes = _ClassDef_merge_classify([l.ClassDef1 for l in lst], allGlyphs=glyphSet)
-	_ClassDef_calculate_Format(self.ClassDef1, merger.font)
+	self.ClassDef1, classes = _ClassDef_merge_classify([l.ClassDef1 for l in lst], allGlyphs=set(self.Coverage.glyphs))
+	_ClassDef_calculate_Format(self.ClassDef1, font)
 	self.Class1Count = len(classes)
 	new_matrices = []
 	for l,matrix in zip(lst, matrices):
@@ -402,9 +387,12 @@ def _PairPosFormat2_merge(self, lst, merger):
 					class2records = nullRow.Class2Record = []
 					# TODO: When merger becomes selfless, revert e6125b353e1f54a0280ded5434b8e40d042de69f
 					for _ in range(l.Class2Count):
-						rec2 = ot.Class2Record()
-						rec2.Value1 = otBase.ValueRecord(l.ValueFormat1) if l.ValueFormat1 else None
-						rec2.Value2 = otBase.ValueRecord(l.ValueFormat2) if l.ValueFormat2 else None
+						if transparent:
+							rec2 = None
+						else:
+							rec2 = ot.Class2Record()
+							rec2.Value1 = otBase.ValueRecord(self.ValueFormat1) if self.ValueFormat1 else None
+							rec2.Value2 = otBase.ValueRecord(self.ValueFormat2) if self.ValueFormat2 else None
 						class2records.append(rec2)
 				rec1 = nullRow
 			else:
@@ -417,7 +405,7 @@ def _PairPosFormat2_merge(self, lst, merger):
 
 	# Align second classes
 	self.ClassDef2, classes = _ClassDef_merge_classify([l.ClassDef2 for l in lst])
-	_ClassDef_calculate_Format(self.ClassDef2, merger.font)
+	_ClassDef_calculate_Format(self.ClassDef2, font)
 	self.Class2Count = len(classes)
 	new_matrices = []
 	for l,matrix in zip(lst, matrices):
@@ -440,21 +428,50 @@ def _PairPosFormat2_merge(self, lst, merger):
 	matrices = new_matrices
 	del new_matrices
 
+	return matrices
+
+def _PairPosFormat2_merge(self, lst, merger):
+	assert _all_equal([l.ValueFormat2 == 0 for l in lst if l.Class1Record]), "Report bug against fonttools."
+
+	merger.mergeObjects(self, lst,
+			    exclude=('Coverage',
+				     'ClassDef1', 'Class1Count',
+				     'ClassDef2', 'Class2Count',
+				     'Class1Record',
+				     'ValueFormat1', 'ValueFormat2'))
+
+	# Align coverages
+	glyphs, _ = _merge_GlyphOrders(merger.font,
+				       [v.Coverage.glyphs for v in lst])
+	self.Coverage.glyphs = glyphs
+
+	# Currently, if the coverage of PairPosFormat2 subtables are different,
+	# we do NOT bother walking down the subtable list when filling in new
+	# rows for alignment.  As such, this is only correct if current subtable
+	# is the last subtable in the lookup.  Ensure that.
+	#
+	# Note that our canonicalization process merges trailing PairPosFormat2's,
+	# so in reality this is rare.
+	for l,subtables in zip(lst,merger.lookup_subtables):
+		if l.Coverage.glyphs != glyphs:
+			assert l == subtables[-1]
+
+	matrices = _PairPosFormat2_align_matrices(self, lst, merger.font)
+
 	self.Class1Record = list(matrices[0]) # TODO move merger to be selfless
 	merger.mergeLists(self.Class1Record, matrices)
 
 @AligningMerger.merger(ot.PairPos)
 def merge(merger, self, lst):
-	# TODO Support differing ValueFormats.
-	merger.valueFormat1 = self.ValueFormat1
-	merger.valueFormat2 = self.ValueFormat2
+	merger.valueFormat1 = self.ValueFormat1 = reduce(int.__or__, [l.ValueFormat1 for l in lst], 0)
+	merger.valueFormat2 = self.ValueFormat2 = reduce(int.__or__, [l.ValueFormat2 for l in lst], 0)
 
 	if self.Format == 1:
 		_PairPosFormat1_merge(self, lst, merger)
 	elif self.Format == 2:
 		_PairPosFormat2_merge(self, lst, merger)
 	else:
-		assert 0
+		assert False
 
 	del merger.valueFormat1, merger.valueFormat2
 
@@ -484,7 +501,7 @@ def merge(merger, self, lst):
 	self.ValueFormat2 = vf2
 
 
-def _PairSet_merge_overlay(lst, font):
+def _PairSet_flatten(lst, font):
 	self = ot.PairSet()
 	self.Coverage = ot.Coverage()
 	self.Coverage.Format = 1
@@ -507,7 +524,9 @@ def _PairSet_merge_overlay(lst, font):
 
 	return self
 
-def _Lookup_PairPosFormat1_subtables_merge_overlay(lst, font):
+def _Lookup_PairPosFormat1_subtables_flatten(lst, font):
+	assert _all_equal([l.ValueFormat2 == 0 for l in lst if l.PairSet]), "Report bug against fonttools."
+
 	self = ot.PairPos()
 	self.Format = 1
 	self.Coverage = ot.Coverage()
@@ -521,45 +540,37 @@ def _Lookup_PairPosFormat1_subtables_merge_overlay(lst, font):
 					    [v.PairSet for v in lst])
 
 	self.Coverage.glyphs = glyphs
-	self.PairSet = [_PairSet_merge_overlay([v for v in values if v is not None], font)
+	self.PairSet = [_PairSet_flatten([v for v in values if v is not None], font)
 		        for values in zip(*padded)]
 	self.PairSetCount = len(self.PairSet)
 	return self
 
-def _Lookup_PairPosFormat2_subtables_recombine(a, b, font):
-	"""Combine two subtables that have the same general structure already."""
+def _Lookup_PairPosFormat2_subtables_flatten(lst, font):
+	assert _all_equal([l.ValueFormat2 == 0 for l in lst if l.Class1Record]), "Report bug against fonttools."
+
 	self = ot.PairPos()
 	self.Format = 2
 	self.Coverage = ot.Coverage()
 	self.Coverage.Format = 1
+	self.ValueFormat1 = reduce(int.__or__, [l.ValueFormat1 for l in lst], 0)
+	self.ValueFormat2 = reduce(int.__or__, [l.ValueFormat2 for l in lst], 0)
+
+	# Align them
 	glyphs, _ = _merge_GlyphOrders(font,
-				       [v.Coverage.glyphs for v in (a, b)])
-
+				       [v.Coverage.glyphs for v in lst])
 	self.Coverage.glyphs = glyphs
-	self.Class1Count = a.Class1Count + b.Class1Count
-	self.ClassDef1 = ot.ClassDef()
 
-	classDefs = self.ClassDef1.classDefs = {}
-	offset = a.Class1Count
-	# First subtable overrides any possible shared glyph, so add b first.
-	sets = _ClassDef_invert(b.ClassDef1, allGlyphs=b.Coverage.glyphs)
-	for i,s in enumerate(sets):
-		for g in s:
-			classDefs[g] = i + offset
-	sets = _ClassDef_invert(a.ClassDef1, allGlyphs=a.Coverage.glyphs)
-	assert len(sets) <= offset
-	for i,s in enumerate(sets):
-		for g in s:
-			classDefs[g] = i
+	matrices = _PairPosFormat2_align_matrices(self, lst, font, transparent=True)
 
-	records = self.Class1Record = []
-	assert a.Class1Count == len(a.Class1Record)
-	assert b.Class1Count == len(b.Class1Record)
-	records.extend(a.Class1Record)
-	records.extend(b.Class1Record)
-
-	for name in ('Class2Count', 'ClassDef2', 'ValueFormat1', 'ValueFormat2'):
-		setattr(self, name, getattr(a, name))
+	matrix = self.Class1Record = []
+	for rows in zip(*matrices):
+		row = ot.Class1Record()
+		matrix.append(row)
+		row.Class2Record = []
+		row = row.Class2Record
+		for cols in zip(*list(r.Class2Record for r in rows)):
+			col = next(iter(c for c in cols if c is not None))
+			row.append(col)
 
 	return self
 
@@ -567,64 +578,76 @@ def _Lookup_PairPos_subtables_canonicalize(lst, font):
 	"""Merge multiple Format1 subtables at the beginning of lst,
 	and merge multiple consecutive Format2 subtables that have the same
 	Class2 (ie. were split because of offset overflows).  Returns new list."""
-	head = []
-	tail = []
-	it = iter(lst)
+	lst = list(lst)
 
-	for subtable in it:
-		if subtable.Format == 1:
-			head.append(subtable)
-			continue
-		tail.append(subtable)
-		break
-	tail.insert(0, _Lookup_PairPosFormat1_subtables_merge_overlay(head, font))
+	l = len(lst)
+	i = 0
+	while i < l and lst[i].Format == 1:
+		i += 1
+	lst[:i] = [_Lookup_PairPosFormat1_subtables_flatten(lst[:i], font)]
 
-	for subtable in it:
-		oldtable = tail[-1]
-		if (oldtable.Format == 2 and subtable.Format == 2 and
-		    oldtable.Class2Count == subtable.Class2Count and
-		    oldtable.ClassDef2.classDefs == subtable.ClassDef2.classDefs and
-		    oldtable.ValueFormat1 == subtable.ValueFormat1 and
-		    oldtable.ValueFormat2 == subtable.ValueFormat2):
-			newtable = _Lookup_PairPosFormat2_subtables_recombine(oldtable, subtable, font)
-			tail[-1] = newtable
-			continue
-		tail.append(subtable)
+	l = len(lst)
+	i = l
+	while i > 0 and lst[i - 1].Format == 2:
+		i -= 1
+	lst[i:] = [_Lookup_PairPosFormat2_subtables_flatten(lst[i:], font)]
 
-	return tail
+	return lst
 
 @AligningMerger.merger(ot.Lookup)
 def merge(merger, self, lst):
-	merger.lookup_subtables = [l.SubTable for l in lst]
+	subtables = merger.lookup_subtables = [l.SubTable for l in lst]
 
-	exclude = []
-	if self.SubTable and isinstance(self.SubTable[0], ot.PairPos):
+	# Remove Extension subtables
+	for l,sts in list(zip(lst,subtables))+[(self,self.SubTable)]:
+		if not sts:
+			continue
+		if sts[0].__class__.__name__.startswith('Extension'):
+			assert _all_equal([st.__class__ for st in sts])
+			assert _all_equal([st.ExtensionLookupType for st in sts])
+			l.LookupType = sts[0].ExtensionLookupType
+			new_sts = [st.ExtSubTable for st in sts]
+			del sts[:]
+			sts.extend(new_sts)
+
+	isPairPos = self.SubTable and isinstance(self.SubTable[0], ot.PairPos)
+
+	if isPairPos:
 
 		# AFDKO and feaLib sometimes generate two Format1 subtables instead of one.
 		# Merge those before continuing.
 		# https://github.com/fonttools/fonttools/issues/719
 		self.SubTable = _Lookup_PairPos_subtables_canonicalize(self.SubTable, merger.font)
-		subtables = [_Lookup_PairPos_subtables_canonicalize(l.SubTable, merger.font) for l in lst]
+		subtables = merger.lookup_subtables = [_Lookup_PairPos_subtables_canonicalize(st, merger.font) for st in subtables]
 
-		merger.lookup_subtables = subtables
-		merger.mergeLists(self.SubTable, subtables)
+	merger.mergeLists(self.SubTable, subtables)
+	self.SubTableCount = len(self.SubTable)
 
+	if isPairPos:
 		# If format-1 subtable created during canonicalization is empty, remove it.
 		assert len(self.SubTable) >= 1 and self.SubTable[0].Format == 1
 		if not self.SubTable[0].Coverage.glyphs:
 			self.SubTable.pop(0)
+			self.SubTableCount -= 1
 
-		self.SubTableCount = len(self.SubTable)
-		exclude.extend(['SubTable', 'SubTableCount'])
+		# If format-2 subtable created during canonicalization is empty, remove it.
+		assert len(self.SubTable) >= 1 and self.SubTable[-1].Format == 2
+		if not self.SubTable[-1].Coverage.glyphs:
+			self.SubTable.pop(-1)
+			self.SubTableCount -= 1
 
-	merger.mergeObjects(self, lst, exclude=exclude)
+	merger.mergeObjects(self, lst, exclude=['SubTable', 'SubTableCount'])
 
 	del merger.lookup_subtables
 
+
 #
 # InstancerMerger
 #
+
 class InstancerMerger(AligningMerger):
+	"""A merger that takes multiple master fonts, and instantiates
+	an instance."""
 
 	def __init__(self, font, model, location):
 		Merger.__init__(self, font)
@@ -638,8 +661,8 @@ def merge(merger, self, lst):
 	YCoords = [a.YCoordinate for a in lst]
 	model = merger.model
 	scalars = merger.scalars
-	self.XCoordinate = round(model.interpolateFromMastersAndScalars(XCoords, scalars))
-	self.YCoordinate = round(model.interpolateFromMastersAndScalars(YCoords, scalars))
+	self.XCoordinate = otRound(model.interpolateFromMastersAndScalars(XCoords, scalars))
+	self.YCoordinate = otRound(model.interpolateFromMastersAndScalars(YCoords, scalars))
 
 @InstancerMerger.merger(otBase.ValueRecord)
 def merge(merger, self, lst):
@@ -655,23 +678,120 @@ def merge(merger, self, lst):
 
 		if hasattr(self, name):
 			values = [getattr(a, name, 0) for a in lst]
-			value = round(model.interpolateFromMastersAndScalars(values, scalars))
+			value = otRound(model.interpolateFromMastersAndScalars(values, scalars))
 			setattr(self, name, value)
 
 
+#
+# MutatorMerger
+#
+
+class MutatorMerger(AligningMerger):
+	"""A merger that takes a variable font, and instantiates
+	an instance."""
+
+	def __init__(self, font, location):
+		Merger.__init__(self, font)
+		self.location = location
+
+		store = None
+		if 'GDEF' in font:
+			gdef = font['GDEF'].table
+			if gdef.Version >= 0x00010003:
+				store = gdef.VarStore
+
+		self.instancer = VarStoreInstancer(store, font['fvar'].axes, location)
+
+	def instantiate(self):
+		font = self.font
+
+		self.mergeTables(font, [font], ['GPOS'])
+
+		if 'GDEF' in font:
+			gdef = font['GDEF'].table
+			if gdef.Version >= 0x00010003:
+				del gdef.VarStore
+				gdef.Version = 0x00010002
+				if gdef.MarkGlyphSetsDef is None:
+					del gdef.MarkGlyphSetsDef
+					gdef.Version = 0x00010000
+			if not (gdef.LigCaretList or
+				gdef.MarkAttachClassDef or
+				gdef.GlyphClassDef or
+				gdef.AttachList or
+				(gdef.Version >= 0x00010002 and gdef.MarkGlyphSetsDef)):
+				del font['GDEF']
+
+@MutatorMerger.merger(ot.Anchor)
+def merge(merger, self, lst):
+	if self.Format != 3:
+		return
+
+	instancer = merger.instancer
+	for v in "XY":
+		tableName = v+'DeviceTable'
+		if not hasattr(self, tableName):
+			continue
+		dev = getattr(self, tableName)
+		delattr(self, tableName)
+		if dev is None:
+			continue
+
+		assert dev.DeltaFormat == 0x8000
+		varidx = (dev.StartSize << 16) + dev.EndSize
+		delta = otRound(instancer[varidx])
+
+		attr = v+'Coordinate'
+		setattr(self, attr, getattr(self, attr) + delta)
+
+	self.Format = 1
+
+@MutatorMerger.merger(otBase.ValueRecord)
+def merge(merger, self, lst):
+
+	# All other structs are merged with self pointing to a copy of base font,
+	# except for ValueRecords which are sometimes created later and initialized
+	# to have 0/None members.  Hence the copy.
+	self.__dict__ = lst[0].__dict__.copy()
+
+	instancer = merger.instancer
+	# TODO Handle differing valueformats
+	for name, tableName in [('XAdvance','XAdvDevice'),
+				('YAdvance','YAdvDevice'),
+				('XPlacement','XPlaDevice'),
+				('YPlacement','YPlaDevice')]:
+
+		if not hasattr(self, tableName):
+			continue
+		dev = getattr(self, tableName)
+		delattr(self, tableName)
+		if dev is None:
+			continue
+
+		assert dev.DeltaFormat == 0x8000
+		varidx = (dev.StartSize << 16) + dev.EndSize
+		delta = otRound(instancer[varidx])
+
+		setattr(self, name, getattr(self, name) + delta)
+
+
 #
 # VariationMerger
 #
 
 class VariationMerger(AligningMerger):
+	"""A merger that takes multiple master fonts, and builds a
+	variable font."""
 
 	def __init__(self, model, axisTags, font):
 		Merger.__init__(self, font)
 		self.model = model
-		self.store_builder = builder.OnlineVarStoreBuilder(axisTags)
+		self.store_builder = varStore.OnlineVarStoreBuilder(axisTags)
 		self.store_builder.setModel(model)
 
 def _all_equal(lst):
+	if not lst:
+		return True
 	it = iter(lst)
 	v0 = next(it)
 	for v in it:
